==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2107_155.zip: ronportal/xml/xmlcontroller.php <==
<?php
/**
 * UTF8
 * stanislav.elbl, 06.11.2015
 */

namespace Ron\RonPortal\XML;

use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\HttpController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Redirect;
use Ron\StringUtils;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\NeniPravoException;
use RonPortal\RonPortalSettings;
use RonPortal\System\PreparedReportOutputObject;
use RonPortal\XML\RonPortalXMLForm;


class XMLFormController extends HttpController
{
    /** @var  Database */
    var $database;

    /**
     * XMLFormController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * Stahnuti dokumentu
     * @return DokumentOutputObject
     * @throws \Exception
     */
    function downloadDokument()
    {
        $idDokumentu = (int)$_GET['record'];
        //if (!$idDokumentu) return false;

        $idXml = (int)$_GET['idxml'];
        //if (!$idXml) return false;

        // kontrola, jestli je dokument k zadanemu formulari pripojen
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT IdDokumentu FROM DokumentXMLFormulare ' .
            ' WHERE IdDokumentu = ' . $params ->addParameter($idDokumentu) .
            ' AND IdFormulare = ' . $params ->addParameter($idXml);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        if (!$row = $cursor ->fetch()) {
            throw new \Exception('Chyba - zadany dokument neni k formulari pripojen!');
        }

        // kontrola prava na xml formular
        $xmlform = new RonPortalXMLForm($this ->database);
        $xmlform->nactiXMLFormular($idXml, false);

        if (!$xmlform->jePravoNaFormular()) {
            throw new NeniPravoException();
        }

        return new DokumentOutputObject($this ->database, $idDokumentu);
    }


    function jsonHttpRequest()
    {
        $result = new JsonOutputObject();
        try {
            $xml_form_id = $_GET['xmlid'];

            require_once('xml/xmlformportal.php');
            $form = new RonPortalXMLForm($this ->database);
            $form ->jsonOutputObject = $result;

            if (isset($_GET['context'])) $form ->context = $_GET['context'];

            $form ->nactiXMLFormular($xml_form_id, true); // zeSablony = true

            $rootElement = $form ->loadXMLElements($form -> xml);

            if ($form ->mode != "json"  &&  !$form ->enableJson) {
                throw new \Exception("Chyba - formular nelze pouzit pro json!");
            }

            $rootElement -> getHTMLOutput(); // vygenerovane HTML nepouzije.  Nebo pouzit process?

            $result ->addMessages(MessageManager::getInstance());

            if ( $form -> xmlResultCode ) {
                $result ->setResult($form -> xmlResultCode);
            }

            if ( $form -> xmlResultMessage ) {
                $result ->addText('resultmessage', $form -> xmlResultMessage);
            }
        } catch (\Exception $e) {
            $result ->addError($e ->getMessage());
        }

        return $result;
    }


    function report()
    {
        if (!$_GET['reportkey']) {
            throw new \Exception('Neni zadan klic sestavy!');
        }

        require_once('adsapp/reportserver.php');
        $format = isset($_GET['format']) ? $_GET['format'] : 'pdf';
        return new PreparedReportOutputObject($_GET['reportkey'], $format);
    }


    public function xmlFormAction()
    {
        switch ($_GET['action']) {
            case 'download' :
                return $this ->downloadDokument();

            case 'json' :
                return $this ->jsonHttpRequest();

            case 'report' :
                return $this ->report();
        }

        throw new \Exception('Chyba v XMLFormController - neznama akce!');
    }


    public function zobrazXmlSablonu($id)
    {
        require_once('xml/xmlform.php');
        return XMLFormViewObject::zobrazXmlSablonu($this ->database, $id);
    }


    public function zobrazVyplnenyFormular($id)
    {
        require_once('xml/xmlform.php');
        return XMLFormViewObject::zobrazVyplnenyXml($this ->database, $id);
//        $result = new XMLFormViewObject($this ->database);
//        return $result ->processXml($id, false);
    }


    public function redirectToSeznam()
    {
        return new Redirect(null,
            '?menuid=seznamxmlformularu' . ($_SESSION['last_xmlmenuid'] ? '&xmlmenuid=' . $_SESSION['last_xmlmenuid'] : '')
        );
    }


    public function ulozFormular()
    {
        // nacte z database XML formular (bud sablonu nebo vyplneny), ktery byl odeslany
        require_once('xml/xmlformportal.php');
        $xmlForm = new RonPortalXMLForm($this->database);
        $xmlForm ->nactiOdeslanyFormular();

        if (isset($_POST['neukladat']) && $_POST['neukladat']) {
            // pouze pro kompatibilitu se starymi formulari, ktere mely neukladat=1 a nemely nastaven mode="..."
            return XMLFormViewObject::showXmlForm($xmlForm);
        }

        if ($xmlForm->readOnly) throw new \Exception( StringUtils::fromUTF8('Chyba! Formulář je pouze pro čtení!') );

        $xmlForm->fill(); // podle mode pouzije bud fill nebo process

        if ($xmlForm->deleted) {   // probehlo v poradku, ale formular byl smazan
            return $this ->redirectToSeznam();
        }

        if ($xmlForm->isError()) {
            // znovu nacte z databaze puvodni XML, aby se zabranilo predavani o 2 urovne a praci s chybne vyplnenym dokumentem
            // POZOR! Musi ale pracovat s puvodnim objektem xmlform, ve kterem jsou napr. nastavene chybove hlasky
            $xmlForm->nactiOdeslanyFormular();

            return XMLFormViewObject::showXmlForm($xmlForm);
        }

        if (!$xmlForm ->mode  &&  $xmlForm ->neukladat) {
            // stary nevyplnovaci formular
            MessageManager::addDebugMessage('XML formular s priznakem "neukladat" je zastaraly! Pouzijte formular s priznakem MODE!');
            return XMLFormViewObject::showXmlForm($xmlForm);
        }

        $newid = $xmlForm->save();

        if (!$newid) {
            throw new \Exception('Pri ukladani formulare doslo k chybe!');
        }

        if ($xmlForm ->redirect) {
            return $xmlForm ->redirect;
        }

        if ($xmlForm->pokracovat  &&  !$xmlForm ->mode) {
            // stary formular bez mode, tag pokracovat zobrazuje formular primo v POST
            return XMLFormViewObject::showXmlForm($xmlForm);
        }

        if (!$xmlForm->viceSekci  &&  !$xmlForm->pokracovat) {
            // v poradku vyplnen, nema vice sekci
            return $this ->redirectToSeznam();
        }


        // formular ma vice sekci, nebo ma nastaven priznak "pokracovat"
        return new Redirect(null, '?menuid=editovatdotaznik&record=' . $newid);
    }


    function processXml()
    {
        if (!isset($_POST['ronxmlform'])  ||  !$_POST['ronxmlform']) {
            return new Redirect(null, '?menuid=xml&record=' . $_GET['record']);
        }

        require_once('xml/xmlformportal.php');
        $xmlForm = new RonPortalXMLForm($this->database);
        $xmlForm ->nactiOdeslanyFormular(); // nacte xml text
        $xmlForm ->loadXML(); // vytvori elementy

        if ($xmlForm ->mode === XMLForm::XML_MODE_SHOW) {
            $xmlForm ->usePostValues = false; // v modu show implicitne nepouziva post hodnoty
            return XMLFormViewObject::showXmlForm($xmlForm);
        }

        $xmlForm ->process(); // zpracuje formular

        if ($xmlForm->isError()) {
            return XMLFormViewObject::showXmlForm($xmlForm);
        }

        if ($xmlForm ->redirect) {
            return $xmlForm ->redirect;
        }

        return new Redirect(null, '?menuid=xml&record=' . $xmlForm ->idSablony);
    }


    public function handleAction($action)
    {
        HtmlHeaders::getInstance() ->addScript('./script/xmlform.js');

        switch ($action) {
            case 'xmlform' :
                return $this ->xmlFormAction();

            case 'xml' :
            case 'vyplnitdotaznik' :
                return (isset($_POST['xmlformpost']) && (int)$_POST['xmlformpost'])
                    ? $this ->processXml()
                    : $this ->zobrazXmlSablonu((int)$_GET['record']);

            case 'editovatdotaznik' :
                return $this ->zobrazVyplnenyFormular((int)$_GET['record']);

            case 'ulozxmlformular' :
                return $this ->ulozFormular();

            case 'processxml' : // Nepouziva se. Jen pro jistotu nechavam pro kompatibilitu pripadnych nestandardnich XML formularu
                return $this ->processXml();
        }

        $xmlformid = RonPortalSettings::getMapovaniXML($action);

        if ($xmlformid) {
            return (isset($_POST['xmlformpost']) && (int)$_POST['xmlformpost'])
                ? $this ->processXml()
                : $this ->zobrazXmlSablonu($xmlformid);
//            $form = new XMLFormular($connection);
//            $form ->zobrazDotaznikZeSablony($xmlformid); // $postvalues = false
//
//            if ($form ->f_mapovanimenu_newmenuid) $action = $form ->f_mapovanimenu_newmenuid;
//            return $form ->f_mapovanimenu_pokracovat;
        }


        throw new \Exception('Chyba v XMLFormController - neznama akce!');
    }

}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2302_169.zip: ronportal/xml/xmlcontroller.php <==
<?php
/**
 * UTF8
 * stanislav.elbl, 06.11.2015
 */

namespace Ron\RonPortal\XML;

use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\HttpController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Redirect;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\NeniPravoException;
use RonPortal\RonPortalSettings;
use RonPortal\System\PreparedReportOutputObject;
use RonPortal\XML\RonPortalXMLForm;
use RonPortal\XML\XMLFormViewObject;


class XMLFormController extends HttpController
{
    /** @var  Database */
    var $database;

    /**
     * XMLFormController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * Stahnuti dokumentu
     * @return DokumentOutputObject
     * @throws \Exception
     */
    function downloadDokument()
    {
        $idDokumentu = (int)$_GET['record'];
        //if (!$idDokumentu) return false;

        $idXml = (int)$_GET['idxml'];
        //if (!$idXml) return false;

        // kontrola, jestli je dokument k zadanemu formulari pripojen
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT IdDokumentu FROM DokumentXMLFormulare ' .
            ' WHERE IdDokumentu = ' . $params ->addParameter($idDokumentu) .
            ' AND IdFormulare = ' . $params ->addParameter($idXml);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        if (!$row = $cursor ->fetch()) {
            throw new \Exception('Chyba - zadany dokument neni k formulari pripojen!');
        }

        // kontrola prava na xml formular
        $xmlform = new RonPortalXMLForm($this ->database);
        $xmlform->nactiXMLFormular($idXml, false);

        if (!$xmlform->jePravoNaFormular()) {
            throw new NeniPravoException();
        }

        return new DokumentOutputObject($this ->database, $idDokumentu);
    }


    function jsonHttpRequest()
    {
        $result = new JsonOutputObject();
        try {
            $xml_form_id = $_GET['xmlid'];

            require_once('xml/xmlformportal.php');
            $form = new RonPortalXMLForm($this ->database);
            $form ->jsonOutputObject = $result;

            if (isset($_GET['context'])) $form ->context = $_GET['context'];

            $form ->nactiXMLFormular($xml_form_id, true); // zeSablony = true

            $rootElement = $form ->loadXMLElements($form -> xml);

            if ($form ->mode != "json"  &&  !$form ->enableJson) {
                throw new \Exception("Chyba - formular nelze pouzit pro json!");
            }

            $rootElement -> getHTMLOutput(); // vygenerovane HTML nepouzije.  Nebo pouzit process?

            $result ->addMessages(MessageManager::getInstance());

            if ( $form -> xmlResultCode ) {
                $result ->setResult($form -> xmlResultCode);
            }

            if ( $form -> xmlResultMessage ) {
                $result ->addText('resultmessage', $form -> xmlResultMessage);
            }
        } catch (\Exception $e) {
            $result ->addError($e ->getMessage());
        }

        return $result;
    }


    function report()
    {
        if (!$_GET['reportkey']) {
            throw new \Exception('Neni zadan klic sestavy!');
        }

        $format = isset($_GET['format']) ? $_GET['format'] : 'pdf';
        return new PreparedReportOutputObject($_GET['reportkey'], $format);
    }


    /**
     * Obsluhuje menuid=xmlform
     * @return JsonOutputObject|DokumentOutputObject|PreparedReportOutputObject
     * @throws NeniPravoException
     * @throws \Exception
     */
    public function xmlFormAction()
    {
        switch ($_GET['action']) {
            case 'download' :
                return $this ->downloadDokument();

            case 'json' :
                return $this ->jsonHttpRequest();

            case 'report' :
                return $this ->report();
        }

        throw new \Exception('Chyba v XMLFormController - neznama akce (parametr action)!');
    }


    public function zobrazXmlSablonu($id)
    {
        require_once('xml/xmlform.php');
        return XMLFormViewObject::zobrazXmlSablonu($this ->database, $id);
    }


    public function zobrazVyplnenyFormular($id)
    {
        require_once('xml/xmlform.php');
        return XMLFormViewObject::zobrazVyplnenyXml($this ->database, $id);
//        $result = new XMLFormViewObject($this ->database);
//        return $result ->processXml($id, false);
    }


    public function redirectToSeznam()
    {
        return new Redirect(null,
            '?menuid=seznamxmlformularu' . ($_SESSION['last_xmlmenuid'] ? '&xmlmenuid=' . $_SESSION['last_xmlmenuid'] : '')
        );
    }


    /**
     * @param  RonPortalXMLForm $xmlForm
     * @return \Ron\MVC\BaseViewObject|Redirect|XMLFormViewObject|string
     * @throws \Exception
     */
    protected function ulozNactenyFormular($xmlForm)
    {
        if (isset($_POST['neukladat']) && $_POST['neukladat']) {
            // pouze pro kompatibilitu se starymi formulari, ktere mely neukladat=1 a nemely nastaven mode="..."
            return XMLFormViewObject::showXmlForm($xmlForm);
        }

        if ($xmlForm->readOnly) {
            throw new \Exception('Chyba! Formular je pouze pro cteni!');
        }

        $xmlForm->fill(); // podle mode pouzije bud fill nebo process

        if ($xmlForm->deleted) {   // probehlo v poradku, ale formular byl smazan
            return $this ->redirectToSeznam();
        }

        if ($xmlForm->isError()) {
            // znovu nacte z databaze puvodni XML, aby se zabranilo predavani o 2 urovne a praci s chybne vyplnenym dokumentem
            // POZOR! Musi ale pracovat s puvodnim objektem xmlform, ve kterem jsou napr. nastavene chybove hlasky
            $xmlForm->nactiOdeslanyFormular();
            $xmlForm->exit = false;

            return XMLFormViewObject::showXmlForm($xmlForm);
        }

        if (!$xmlForm ->mode  &&  $xmlForm ->neukladat) {
            // stary nevyplnovaci formular
            MessageManager::addDebugMessage('XML formular s priznakem "neukladat" je zastaraly! Pouzijte formular s priznakem MODE!');
            return XMLFormViewObject::showXmlForm($xmlForm);
        }

        $newid = $xmlForm->save();

        if (!$newid) {
            throw new \Exception('Pri ukladani formulare doslo k chybe!');
        }

        if ($xmlForm ->redirect) {
            return $xmlForm ->redirect;
        }

        if ($xmlForm->pokracovat  &&  !$xmlForm ->mode) {
            // stary formular bez mode, tag pokracovat zobrazuje formular primo v POST
            return XMLFormViewObject::showXmlForm($xmlForm);
        }

        if (!$xmlForm->viceSekci  &&  !$xmlForm->pokracovat) {
            // v poradku vyplnen, nema vice sekci
            return $this ->redirectToSeznam();
        }

        // formular ma vice sekci, nebo ma nastaven priznak "pokracovat"
        return new Redirect(null, '?menuid=editovatdotaznik&record=' . $newid);
    }

    public function ulozFormular()
    {
        // nacte z database XML formular (bud sablonu nebo vyplneny), ktery byl odeslany
        require_once('xml/xmlformportal.php');
        $xmlForm = new RonPortalXMLForm($this->database);
        $xmlForm ->nactiOdeslanyFormular();

        return $this ->ulozNactenyFormular($xmlForm);
    }


    function processXml()
    {
        if (!isset($_POST['ronxmlform'])  ||  !$_POST['ronxmlform']) {
            return new Redirect(null, '?menuid=xml&record=' . $_GET['record']);
        }

        require_once('xml/xmlformportal.php');
        $xmlForm = new RonPortalXMLForm($this->database);
        $xmlForm ->nactiOdeslanyFormular(); // nacte xml text
        $xmlForm ->loadXML(); // vytvori elementy

        if ($xmlForm ->mode === XMLForm::XML_MODE_SHOW) {
            $xmlForm ->usePostValues = false; // v modu show implicitne nepouziva post hodnoty
            return XMLFormViewObject::showXmlForm($xmlForm);
        }

        if ($xmlForm ->mode === XMLForm::XML_MODE_DATA) {
            return $this ->ulozNactenyFormular($xmlForm);
        }

        $xmlForm ->process(); // zpracuje formular

        if ($xmlForm->isError()) {
            $xmlForm ->exit = false;
            return XMLFormViewObject::showXmlForm($xmlForm);
        }

        if ($xmlForm ->redirect) {
            return $xmlForm ->redirect;
        }

        return new Redirect(null, '?menuid=xml&record=' . $xmlForm ->idSablony);
    }


    public function handleAction($action)
    {
        HtmlHeaders::getInstance() ->addScript('./script/xmlform.js');

        switch ($action) {
            case 'xmlform' :
                return $this ->xmlFormAction();

            case 'xml' :
            case 'vyplnitdotaznik' :
                return (isset($_POST['xmlformpost']) && (int)$_POST['xmlformpost'])
                    ? $this ->processXml()
                    : $this ->zobrazXmlSablonu((int)$_GET['record']);

            case 'editovatdotaznik' :
                return (isset($_POST['xmlformpost']) && (int)$_POST['xmlformpost'])
                    ? $this ->ulozFormular()
                    : $this ->zobrazVyplnenyFormular((int)$_GET['record']);

            case 'ulozxmlformular' :  // jen pro stare XML (XSL)
                return $this ->ulozFormular();

            case 'processxml' : // Nepouziva se. Jen pro jistotu nechavam pro kompatibilitu pripadnych nestandardnich XML formularu
                return $this ->processXml();
        }

        $xmlformid = RonPortalSettings::getMapovaniXML($action);

        if ($xmlformid) {
            return (isset($_POST['xmlformpost']) && (int)$_POST['xmlformpost'])
                ? $this ->processXml()
                : $this ->zobrazXmlSablonu($xmlformid);
//            $form = new XMLFormular($connection);
//            $form ->zobrazDotaznikZeSablony($xmlformid); // $postvalues = false
//
//            if ($form ->f_mapovanimenu_newmenuid) $action = $form ->f_mapovanimenu_newmenuid;
//            return $form ->f_mapovanimenu_pokracovat;
        }


        throw new \Exception('Chyba v XMLFormController - neznama akce!');
    }

}
